@@ -1,10 +1,11 @@
-// { dg-require-namedlocale "" }
+// { dg-require-namedlocale "en_US" }
 // { dg-require-fork "" }
 // { dg-require-mkfifo "" }
 
-// 2004-01-11  Petur Runolfsson  <peturr02@ru.is>
+// 2004-04-16  Petur Runolfsson  <peturr02@ru.is>
 
-// Copyright (C) 2004, 2005, 2006, 2007, 2009 Free Software Foundation, Inc.
+// Copyright (C) 2004, 2005, 2006, 2007, 2009, 2010
+// Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -21,65 +22,74 @@
 // with this library; see the file COPYING3.  If not see
 // <http://www.gnu.org/licenses/>.
 
-// 27.8.1.4 Overridden virtual functions
-
 #include <fstream>
 #include <locale>
 #include <cstdlib>
-
 #include <unistd.h>
+#include <signal.h>
+#include <fcntl.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 
+// No asserts, avoid leaking the semaphore if a VERIFY fails.
+#undef _GLIBCXX_ASSERT
+
 #include <testsuite_hooks.h>
 
-// libstdc++/13582
-void test01()
+// libstdc++/14975
+bool test01()
 {
-  bool test __attribute__((unused)) = true;
   using namespace std;
   using namespace __gnu_test;
+  bool test __attribute__((unused)) = true;
 
-  locale loc_en(locale("en_US"));
-  locale loc_fr(locale("fr_FR"));
+  locale loc_us = locale("en_US");
 
-  const char* name = "tmp_fifo_13582-2";
-  unlink(name);
+  const char* name = "tmp_14975-2";
+
+  signal(SIGPIPE, SIG_IGN);
+
+  unlink(name);  
   mkfifo(name, S_IRWXU);
-  
+  semaphore s1;
+
   int child = fork();
+  VERIFY( child != -1 );
+
   if (child == 0)
     {
-      filebuf fbout;
-      fbout.open(name, ios_base::out);
-      fbout.sputn("12345", 5);
-      fbout.pubsync();
-      sleep(2);
-      fbout.close();
+      {
+	filebuf fbin;
+	fbin.open(name, ios_base::in);
+      }
+      s1.signal();
       exit(0);
     }
+  
+  wfilebuf fb;
+  fb.pubimbue(loc_us);
+  wfilebuf* ret = fb.open(name, ios_base::out);
+  VERIFY( ret != 0 );
+  VERIFY( fb.is_open() );
 
-  wfilebuf fbin;
-  fbin.open(name, ios_base::in);
-  sleep(1);
-  wfilebuf::int_type n = fbin.sbumpc();
-  VERIFY( n == L'1' );
-  fbin.pubimbue(loc_en);
-  n = fbin.sbumpc();
-  VERIFY( n == L'2' );
-  fbin.pubimbue(loc_fr);
-  n = fbin.sbumpc();
-  VERIFY( n == L'3' );
-  n = fbin.sbumpc();
-  VERIFY( n == L'4' );
-  n = fbin.sbumpc();
-  VERIFY( n == L'5' );
-  n = fbin.sbumpc();
-  VERIFY( n == wfilebuf::traits_type::eof() );
+  s1.wait();
+
+  try
+    {
+      fb.sputc(L'a');
+      fb.sputc(L'b');
+      fb.pubimbue(locale::classic());
+      fb.sputc(L'c');
+      fb.close();
+    }
+  catch (std::exception&)
+    {
+    }
+
+  return test;
 }
 
 int main()
 {
-  test01();
-  return 0;
+  return !test01();
 }
