@@ -3,7 +3,7 @@
 // { dg-require-cstdint "" }
 // { dg-require-gthreads "" }
 
-// Copyright (C) 2008, 2009 Free Software Foundation, Inc.
+// Copyright (C) 2008, 2009, 2010 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -29,8 +29,7 @@ void test01()
   typedef std::recursive_timed_mutex mutex_type;
   mutex_type m1;
   mutex_type m2;
-  m1 = m2;
+  m1 = m2;			// { dg-error "deleted" }
 }
 
-// { dg-error "used here" "" { target *-*-* } 32 } 
-// { dg-error "deleted function" "" { target *-*-* } 298 }
+// { dg-prune-output "include" }
